' Read all 19 registers of RTC DS3231
' Bob D 2014 03 11

' DS3231 address
RTCaddr = &h68
' DS3231 register 0
RegAddr0 = 0

Dim RTCdata(19) Length 2

I2Cen 100,100 
I2Csend RTCaddr, 0,1,RegAddr0
If MM.I2C <> 0 then 
  print "I2C error = "; MM.I2C
  end
Endif
I2Crcv RTCaddr, 0, 19, RTCdata(0)
If MM.I2C <> 0 then 
  print "I2C error = "; MM.I2C
  end
Endif
I2Cdis

' Convert time and date data
' the data in the registers is BCD (aka Packed Decimal) format
' with a decimal number from 0 to 9 in each half byte (nibble)
' each byte has a 10s decimal and a units decimal
' except day of the week which is units only
' Not all bits are used in each half byte - only what is needed for that data
' Example: Day of month range is 1 to 31 so we need 2 bits for the 10s (decimal 0 to 3)
' and 4 bits for the units (decimal 0 to 9) so we can then express 
' all decimal numbers from 1 to 31 in a byte
' each half byte needs to be processed 4 bits at a time and then added together

Seconds = (RTCdata(0) and &h70)\16*10 + (RTCdata(0) and &h0f) 
Minutes = (RTCdata(1) and &h70)\16*10 + (RTCdata(1) and &h0f) 
Hours =   (RTCdata(2) and &h70)\16*10 + (RTCdata(2) and &h0f) 
DoW =                                   (RTCdata(3) and &h07)
DoM =     (RTCdata(4) and &h30)\16*10 + (RTCdata(4) and &h0f)
Month =   (RTCdata(5) and &h10)\16*10 + (RTCdata(5) and &h0f)
Year =    (RTCdata(6) and &hf0)\16*10 + (RTCdata(6) and &h0f) + 2000

T$ = str$(Hours)+":"+str$(Minutes)+":"+str$(Seconds)
D$ = str$(DoM)+"-"+str$(Month)+"-"+str$(Year)
Time$ = T$
Date$ = D$

' from here is optional but is dependent on previous code

' Convert Day of the week to DayName - Sunday=1 - Saturday=7
DayNames$ = "123456789Sunday   6Monday   6Tuesday  7Wednesday9Thursday 8Friday   6Saturday 8"
DayName$ = Mid$(DayNames$, DoW*10, 10)
L = Val(Right$(DayName$, 1))
DayName$ = Left$(DayName$,L)

' Convert Month number to MonthName - January=1 - December=12
MonthNames$ = "123456789January  7February 8March    5April    5May      3June     4July     4August   6September9October  7November 8December 8"
MonthName$ = Mid$(MonthNames$, Month*10, 10)
L = Val(Right$(MonthName$, 1))
MonthName$ = Left$(MonthName$,L)

Print "The time is "; Time$; " on ";DayName$;DoM;" ";MonthName$;Year

' Convert temperature and Aging Offset data
' Both data are in Twos Complement format
 
If RTCdata(17) < &h80 Then
  Temperature = RTCdata(17) + (RTCdata(18) / 256)
  Else
  Temperature = -(((RTCdata(17) Xor &hFF) + 1) + (RTCdata(18) \ 256))
EndIf
t$=Format$(Temperature,"%+6.2f")
Print "Temperature is "; T$; " degrees Celsius"

If RTCdata(16) < &h80 then 
   Offset = RTCdata(16)
   Else
   Offset = -(RTCdata(16) Xor &hFF)
Endif
Print "The Aging Offset data is "; Offset

End

